package org.processmining.acceptingpetrinetdecomposer.algorithms;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinetdecomposer.parameters.ValidateActivityClusterArrayParameters;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarray.models.impl.ActivityClusterArrayFactory;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;

public class ValidateActivityClusterArrayAlgorithm {

	public ActivityClusterArray apply(PluginContext context, ActivityClusterArray clusters,
			AcceptingPetriNetArray nets, ValidateActivityClusterArrayParameters parameters) {
		ActivityClusterArray validatedClusters = ActivityClusterArrayFactory.createActivityClusterArray();
		validatedClusters.init(clusters);
		Set<Integer> invalidIndices = new HashSet<Integer>();
		for (int i = 0; i < clusters.getClusters().size(); i++) {
			if (!invalidIndices.contains(i)) {
				Set<XEventClass> cluster = new HashSet<XEventClass>(clusters.getCluster(i));
				Set<String> placeIds = getPlaceIds(nets.getNet(i));
				boolean progress = true;
				while (progress) {
					progress = false;
					for (int j = i + 1; j < nets.getSize(); j++) {
						if (!invalidIndices.contains(j)) {
							Set<String> otherPlaceIds = getPlaceIds(nets.getNet(j));
							otherPlaceIds.retainAll(placeIds);
							if (!otherPlaceIds.isEmpty()) {
								progress = true;
								invalidIndices.add(j);
								placeIds.addAll(getPlaceIds(nets.getNet(j)));
								cluster.addAll(clusters.getCluster(j));
							}
						}
					}
				}
				validatedClusters.addCluster(cluster);
			}
		}
		return validatedClusters;
	}

	private Set<String> getPlaceIds(AcceptingPetriNet net) {
		Set<String> placeIds = new HashSet<String>();
		for (Place place : net.getNet().getPlaces()) {
			placeIds.add(place.getLabel());
		}
		return placeIds;
	}
}
