package org.processmining.acceptingpetrinetdecomposer.algorithms.sese;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinetdecomposer.models.rpst.PetriNetRPST;
import org.processmining.acceptingpetrinetdecomposer.models.rpst.PetriNetRPSTNode;
import org.processmining.acceptingpetrinetdecomposer.parameters.sese.DecomposeBySESEsWithBridgingParameters;
import org.processmining.acceptingpetrinetdecomposer.plugins.sese.GenerateRPSTFromPetriNetPlugin;
import org.processmining.acceptingpetrinetdecomposer.utils.dc.BridgingUtils;
import org.processmining.acceptingpetrinetdecomposer.utils.dc.PartitioningUtils;
import org.processmining.acceptingpetrinetdecomposer.utils.petrinet.AcceptingPetriNetArrayUtils;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class DecomposeBySESEsWithBridgingAlgorithm {

	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet apn,
			DecomposeBySESEsWithBridgingParameters parameters) {
		
		// Generate the RPST
		PetriNetRPST rpst = generateRPST(context, apn);
		context.getProvidedObjectManager().createProvidedObject(rpst.getName(), rpst, 
				PetriNetRPST.class, context);
		
		// Make a K-partition over the RPST
		List<PetriNetRPSTNode> partNodes = partitionNodes(rpst, parameters.getMaxSize(), parameters.getUnsplittables());
		
		// Bridging (create explicit separate nets for the common places among nets)
		List<Collection<PetrinetEdge<?, ?>>> decomposition = bridgeNodes(partNodes, apn);
		
		List<Map<PetrinetNode, PetrinetNode>> parentChildMaps = new ArrayList<>();
		
		return AcceptingPetriNetArrayUtils.getNets(apn, decomposition, parentChildMaps);
	}
	
	private PetriNetRPST generateRPST(PluginContext context, AcceptingPetriNet apn) {
		GenerateRPSTFromPetriNetPlugin generateRPSTPlugin = new GenerateRPSTFromPetriNetPlugin();
		PetriNetRPST rpst = generateRPSTPlugin.runDefault(context, apn);
		return rpst;
		
	}
	
	private List<PetriNetRPSTNode> partitionNodes(PetriNetRPST rpst, int maxSize, Set<Transition> unsplittables) {
		List<PetriNetRPSTNode> partNodes = PartitioningUtils.partitioning(rpst, maxSize, unsplittables);
		return partNodes;
	}
	
	private List<Collection<PetrinetEdge<?, ?>>> bridgeNodes(List<PetriNetRPSTNode> partNodes,
			AcceptingPetriNet apn) {
		return BridgingUtils.bridging(partNodes, apn);
	}
}
