package org.processmining.acceptingpetrinetdecomposer.dialogs;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinetdecomposer.parameters.DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters;
import org.processmining.acceptingpetrinetdecomposer.strategies.DecompositionStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.DecompositionStrategyManager;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionSESEStrategy;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.pnetreplayer.dialogs.TransEvClassMappingPanel;

import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

public class DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6732517188360796457L;

	public DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog(AcceptingPetriNet net,
			ActivityClusterArray clusters, final DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		this(net, getActivities(clusters), parameters);
	}
	
	public DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog(AcceptingPetriNet net,
			Set<XEventClass> activities, final DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		double size[][] = { { TableLayoutConstants.FILL, TableLayoutConstants.FILL, TableLayoutConstants.FILL }, 
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL,  30 } };
		setLayout(new TableLayout(size));
		add(new TransEvClassMappingPanel(activities, parameters), "0, 0, 1, 1");

		final JCheckBox includeBox = SlickerFactory.instance().createCheckBox("Include all cluster transitions", false);
		includeBox.setSelected(parameters.isIncludeClusterTransitions());
		includeBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				parameters.setIncludeClusterTransitions(includeBox.isSelected());
			}

		});
		includeBox.setOpaque(false);
		add(includeBox, "0, 2");

		// adding JPanel for configuring max arcs per subnet for SESE decomposition
		int minNumOfArcs = 1;
		int maxNumOfArcs = net.getNet().getEdges().size();
		final ProMTextField maxArcsTextField = makeMaxArcsTextField(minNumOfArcs, maxNumOfArcs, parameters);
		final JPanel maxArcsJPanel = makeMaxArcsJPanel(maxArcsTextField, minNumOfArcs, maxNumOfArcs);
		add(maxArcsJPanel, "1, 2, 2, 2");
		
		DefaultListModel<String> listModel = new DefaultListModel<String>();
		for (DecompositionStrategy strategy : DecompositionStrategyManager.getInstance().getStrategies()) {
			listModel.addElement(strategy.getName());
		}
		final ProMList<String> list = new ProMList<String>("Select decomposition strategy", listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		final String defaultStrategy = parameters.getStrategy();
		list.setSelection(defaultStrategy);
		list.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				List<String> selected = list.getSelectedValuesList();
				if (selected.size() == 1) {
					String selectedStrategy = selected.get(0);
					parameters.setStrategy(selectedStrategy);
					// need to enable the max arc text field if it is SESE-based decomposition
					if (selectedStrategy.equals(DecompositionSESEStrategy.NAME)) {
						maxArcsTextField.setEditable(true);
						maxArcsTextField.setEnabled(true);
					}
				} else {
					/*
					 * Nothing selected. Revert to selection of default
					 * classifier.
					 */
					list.setSelection(defaultStrategy);
					parameters.setStrategy(defaultStrategy);
				}
			}
		});
		list.setPreferredSize(new Dimension(100, 100));
		add(list, "2, 0");
		
		DefaultListModel<XEventClass> activityModel = new DefaultListModel<XEventClass>();
		for (XEventClass activity : activities) {
			activityModel.addElement(activity);
		}
		final ProMList<XEventClass> activityList = new ProMList<XEventClass>("Select unsplittable activities", activityModel);
		activityList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		final Set<XEventClass> defaultUnsplittableActivities = parameters.getUnsplittableActivities();
		activityList.setSelection(defaultUnsplittableActivities);
		activityList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				parameters.setUnsplittableActivities(new HashSet<XEventClass>(activityList.getSelectedValuesList()));
			}
		});
		activityList.setPreferredSize(new Dimension(100, 100));
		add(activityList, "2, 1, 2, 1");
	}

	private static Set<XEventClass> getActivities(ActivityClusterArray clusters) {
		Set<XEventClass> activities = new HashSet<XEventClass>();
		for (Set<XEventClass> cluster : clusters.getClusters()) {
			activities.addAll(cluster);
		}
		return activities;
	}

	private ProMTextField makeMaxArcsTextField(final int minNumOfArcs, final int maxNumOfArcs, 
			final DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		final ProMTextField maxArcsTextField = new ProMTextField(parameters.getMaxSubnetArcs() + "");
		maxArcsTextField.setText(parameters.getMaxSubnetArcs() + "");
		maxArcsTextField.setEditable(false);
		maxArcsTextField.setEnabled(false);
		maxArcsTextField.getDocument().addDocumentListener(new DocumentListener() {
			
			private int parseTextAsInt (String s) {
				try {
					int parsed = Integer.parseInt(s);
					// need to check that it is within the required range
					if (parsed >= minNumOfArcs && parsed <= maxNumOfArcs) 
						return parsed;
				} catch (NumberFormatException nfe) {
					
				}
				// default to not changing
				return parameters.getMaxSubnetArcs();
			}
			
			private void updateMaxArcsInParam(final DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
				int newValue = parseTextAsInt(maxArcsTextField.getText());
				System.out.println("[DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog] "
						+ "Updating max arcs per subnet to: " + newValue);
				parameters.setMaxSubnetArcs(newValue);
			}
			
			public void removeUpdate(DocumentEvent e) {
				updateMaxArcsInParam(parameters);
			}
			
			public void insertUpdate(DocumentEvent e) {
				updateMaxArcsInParam(parameters);			
			}
			
			public void changedUpdate(DocumentEvent e) {
				updateMaxArcsInParam(parameters);			
			}
		});
		if (parameters.getStrategy().equals(DecompositionSESEStrategy.NAME)) {
			maxArcsTextField.setEditable(true);
			maxArcsTextField.setEnabled(true);
		}
		maxArcsTextField.setPreferredSize(new Dimension(100, 20));
		return maxArcsTextField;
	}

	private JPanel makeMaxArcsJPanel(final ProMTextField maxArcsTextField, final int minNumOfArcs, final int maxNumOfArcs) {
		final JPanel maxArcsJPanel = new JPanel();
		maxArcsJPanel.setOpaque(false);
		final JLabel maxArcsJLabel = new JLabel("Max arcs per subnet (min: " + minNumOfArcs + ", max: " + maxNumOfArcs + ")");
		maxArcsJLabel.setOpaque(false);
		maxArcsJPanel.add(maxArcsJLabel, "0, 0");
		maxArcsJPanel.add(maxArcsTextField, "1, 0");
		return maxArcsJPanel;
	}
	
}
