package org.processmining.acceptingpetrinetdecomposer.plugins;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinetdecomposer.algorithms.DecomposeAcceptingPetriNetUsingActivityClusterArrayAlgorithm;
import org.processmining.acceptingpetrinetdecomposer.connections.DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection;
import org.processmining.acceptingpetrinetdecomposer.dialogs.DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog;
import org.processmining.acceptingpetrinetdecomposer.parameters.DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Split Accepting Petri Net", parameterLabels = { "Accepting Petri Net", "Activity Cluster Array",
		"Parameters" }, returnLabels = { "Accepting Petri Net Array" }, returnTypes = { AcceptingPetriNetArray.class })
public class DecomposeAcceptingPetriNetUsingActivityClusterArrayPlugin extends
		DecomposeAcceptingPetriNetUsingActivityClusterArrayAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Split Accepting Petri Net, UI", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNetArray runUI(UIPluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters) {
		DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters = new DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters(
				acceptingNet, clusters, null);
		DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog dialog = new DecomposeAcceptingPetriNetUsingActivityClusterArrayDialog(
				acceptingNet, clusters, parameters);
		InteractionResult result = context.showWizard("Configure split (transition-activity map)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, acceptingNet, clusters, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public AcceptingPetriNetArray decomposeUI(UIPluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters) {
		return runUI(context, acceptingNet, clusters);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Split Accepting Petri Net, Default", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNetArray runDefault(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters) {
		Set<XEventClass> activities = new HashSet<XEventClass>();
		for (Set<XEventClass> cluster : clusters.getClusters()) {
			activities.addAll(cluster);
		}
		DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters = new DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters(
				acceptingNet, activities, null);
		return runConnection(context, acceptingNet, clusters, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNetArray decomposeDefault(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters) {
		return runDefault(context, acceptingNet, clusters);
	}

	@PluginVariant(variantLabel = "Split Accepting Petri Net, Parameters", requiredParameterLabels = { 0, 1, 2 })
	public AcceptingPetriNetArray run(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		return runConnection(context, acceptingNet, clusters, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNetArray decomposeParameters(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		return runConnection(context, acceptingNet, clusters, parameters);
	}

	private AcceptingPetriNetArray runConnection(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection.class, context, acceptingNet);
				for (DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection connection : connections) {
					if (connection.getObjectWithRole(DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection.NET)
							.equals(acceptingNet)
							&& connection.getObjectWithRole(
									DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection.CLUSTERS).equals(
									clusters) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection.NETS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNetArray acceptingNets = apply(context, acceptingNet, clusters, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new DecomposeAcceptingPetriNetUsingActivityClusterArrayConnection(acceptingNet, acceptingNets,
							clusters, parameters));
		}
		return acceptingNets;
	}

}
