package org.processmining.acceptingpetrinetdecomposer.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinetdecomposer.algorithms.ValidateActivityClusterArrayAlgorithm;
import org.processmining.acceptingpetrinetdecomposer.connections.ValidateActivityClusterArrayConnection;
import org.processmining.acceptingpetrinetdecomposer.help.ValidateActivityClusterArrayHelp;
import org.processmining.acceptingpetrinetdecomposer.parameters.ValidateActivityClusterArrayParameters;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Validate Clusters", parameterLabels = { "Activity Cluster Array", "Accepting Petri Net" }, returnLabels = { "Valid Activity Cluster Array" }, returnTypes = { ActivityClusterArray.class }, help = ValidateActivityClusterArrayHelp.TEXT)
public class ValidateActivityClusterArrayPlugin extends ValidateActivityClusterArrayAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Validate Clusters, Default", requiredParameterLabels = { 0, 1 })
	public ActivityClusterArray runDefault(PluginContext context, ActivityClusterArray clusters,
			AcceptingPetriNetArray nets) {
		ValidateActivityClusterArrayParameters parameters = new ValidateActivityClusterArrayParameters();
		return runConnection(context, clusters, nets, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public ActivityClusterArray validateDefault(PluginContext context, ActivityClusterArray clusters,
			AcceptingPetriNetArray nets) {
		return runDefault(context, clusters, nets);
	}

	@PluginVariant(variantLabel = "Validate Clusters, Parameters", requiredParameterLabels = { 0, 1 })
	public ActivityClusterArray run(PluginContext context, ActivityClusterArray clusters,
			AcceptingPetriNetArray nets, ValidateActivityClusterArrayParameters parameters) {
		return runConnection(context, clusters, nets, parameters);
	}
	
	private ActivityClusterArray runConnection(PluginContext context, ActivityClusterArray clusters,
			AcceptingPetriNetArray nets, ValidateActivityClusterArrayParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ValidateActivityClusterArrayConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						ValidateActivityClusterArrayConnection.class, context, clusters);
				for (ValidateActivityClusterArrayConnection connection : connections) {
					if (connection.getObjectWithRole(ValidateActivityClusterArrayConnection.CLUSTERS).equals(clusters)
							&& connection.getObjectWithRole(ValidateActivityClusterArrayConnection.NETS).equals(nets)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(ValidateActivityClusterArrayConnection.VALIDATEDCLUSTERS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		ActivityClusterArray validatedClusters = apply(context, clusters, nets, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new ValidateActivityClusterArrayConnection(clusters, nets, validatedClusters, parameters));
		}
		return validatedClusters;
	}

}
