package org.processmining.acceptingpetrinetdecomposer.strategies;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionGenericFiftyStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionGenericHundredStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionGenericSeventyFiveStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionGenericStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionReplaceNoReduceStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionReplaceReduceStrategy;
import org.processmining.acceptingpetrinetdecomposer.strategies.impl.DecompositionSESEStrategy;

public class DecompositionStrategyManager {

	private static DecompositionStrategyManager instance = null;
	private List<DecompositionStrategy> strategies;
	private DecompositionStrategy defaultStrategy;
	private boolean sorted;

	private DecompositionStrategyManager() {
		strategies = new ArrayList<DecompositionStrategy>();
		defaultStrategy = null;
		sorted = false;
//		register(new DecompositionFilterStrategy(), false);
		register(new DecompositionReplaceReduceStrategy(), false);
		register(new DecompositionReplaceNoReduceStrategy(), false);
		register(new DecompositionGenericStrategy(), true);
		register(new DecompositionGenericHundredStrategy(), false);
		register(new DecompositionGenericSeventyFiveStrategy(), false);
		register(new DecompositionGenericFiftyStrategy(), false);
		register(new DecompositionSESEStrategy(), false);
	}

	public static DecompositionStrategyManager getInstance() {
		if (instance == null) {
			instance = new DecompositionStrategyManager();
		}
		return instance;
	}
	
	public void register(DecompositionStrategy strategy, boolean isDefault) {
		strategies.add(strategy);
		if (isDefault) {
			defaultStrategy = strategy;
		}
		sorted = false;
	}

	public List<DecompositionStrategy> getStrategies() {
		if (!sorted) {
			Collections.sort(strategies, new Comparator<DecompositionStrategy>() {
				public int compare(DecompositionStrategy m1, DecompositionStrategy m2) {
					return m1.getName().compareTo(m2.getName());
				}
			});
			sorted = true;
		}
		return strategies;
	}
	
	public boolean isDefault(DecompositionStrategy strategy) {
		return strategy == defaultStrategy;
	}
	
	public DecompositionStrategy getStrategy(String name) {
		if (name != null) {
			for (DecompositionStrategy strategy : strategies) {
				if (name.equals(strategy.getName())) {
					return strategy;
				}
			}
		}
		return defaultStrategy;
	}
}
