package org.processmining.acceptingpetrinetdecomposer.strategies.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetArrayFactory;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinetdecomposer.parameters.DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters;
import org.processmining.acceptingpetrinetdecomposer.strategies.DecompositionStrategy;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.log.utils.XUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

public abstract class DecompositionAbstractGenericStrategy implements DecompositionStrategy {

	public abstract int getPercentage(DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters);

	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		return apply(context, acceptingNet, clusters, parameters, new ArrayList<Map<PetrinetNode, PetrinetNode>>());
	}

	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters,
			List<Map<PetrinetNode, PetrinetNode>> parentChildMap) {
		Collection<Collection<PetrinetEdge<?, ?>>> decomposition = getMaximalDecomposition(acceptingNet, parameters);
		decomposition = getNonMaximalDecomposition(acceptingNet, decomposition, parameters);
		parameters.displayMessage("[DecompositionGenericStrategy] Decomposed net to " + decomposition.size()
				+ " subnets.");
		return getNets(acceptingNet, decomposition, parameters, parentChildMap);
	}

	private Collection<Collection<PetrinetEdge<?, ?>>> getMaximalDecomposition(AcceptingPetriNet acceptingNet,
			DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		Petrinet net = acceptingNet.getNet();
		TransEvClassMapping tecMap = parameters.getMapping();
		XEventClass invisible = parameters.getInvisibleActivity();
		Collection<Collection<PetrinetEdge<?, ?>>> decomposition = new HashSet<Collection<PetrinetEdge<?, ?>>>();
		/*
		 * Create an initial cluster for every place.
		 */
		//		System.out.println("[DecompositionAbstractGenericStrategy] Clustering places.");
		for (Place place : net.getPlaces()) {
			Collection<PetrinetEdge<?, ?>> cluster = new HashSet<PetrinetEdge<?, ?>>();
			for (PetrinetEdge<?, ?> edge : net.getInEdges(place)) {
				cluster.add(edge);
			}
			for (PetrinetEdge<?, ?> edge : net.getOutEdges(place)) {
				cluster.add(edge);
			}
			decomposition.add(cluster);
		}
		/*
		 * Merge clusters that are mapped onto the invisible activity.
		 */
		//		System.out.println("[DecompositionAbstractGenericStrategy] Clustering invisible transitions.");
		for (Transition transition : net.getTransitions()) {
			if (tecMap.get(transition).equals(invisible)) {
				Collection<PetrinetEdge<?, ?>> newCluster = new HashSet<PetrinetEdge<?, ?>>();
				Collection<Collection<PetrinetEdge<?, ?>>> oldClusters = new HashSet<Collection<PetrinetEdge<?, ?>>>();
				for (PetrinetEdge<?, ?> edge : net.getInEdges(transition)) {
					for (Collection<PetrinetEdge<?, ?>> cluster : decomposition) {
						if (cluster.contains(edge)) {
							newCluster.addAll(cluster);
							oldClusters.add(cluster);
						}
					}
				}
				for (PetrinetEdge<?, ?> edge : net.getOutEdges(transition)) {
					for (Collection<PetrinetEdge<?, ?>> cluster : decomposition) {
						if (cluster.contains(edge)) {
							newCluster.addAll(cluster);
							oldClusters.add(cluster);
						}
					}
				}
				decomposition.removeAll(oldClusters);
				decomposition.add(newCluster);
			}
		}
		/*
		 * Get the activities for which there are multiple transitions.
		 */
		//		System.out.println("[DecompositionAbstractGenericStrategy] Clustering duplicate transitions.");
		Collection<XEventClass> seenActivities = new HashSet<XEventClass>();
		Collection<XEventClass> duplicateActivities = new HashSet<XEventClass>(parameters.getUnsplittableActivities());
		for (Transition transition : net.getTransitions()) {
			if (!tecMap.get(transition).equals(invisible)) {
				XEventClass activity = tecMap.get(transition);
				if (activity.equals(XUtils.MOVEONMODELACTIVITY)) {
					activity = new XEventClass(transition.getLabel(), 0);
				}
				if (seenActivities.contains(activity)) {
					duplicateActivities.add(activity);
				}
				seenActivities.add(activity);
			}
		}
		/*
		 * Merge clusters with duplicate activities.
		 */
		for (XEventClass duplicateActivity : duplicateActivities) {
			Collection<PetrinetEdge<?, ?>> newCluster = new HashSet<PetrinetEdge<?, ?>>();
			Collection<Collection<PetrinetEdge<?, ?>>> oldClusters = new HashSet<Collection<PetrinetEdge<?, ?>>>();
			for (Transition transition : net.getTransitions()) {
				XEventClass activity = tecMap.get(transition);
				if (activity.equals(XUtils.MOVEONMODELACTIVITY)) {
					activity = new XEventClass(transition.getLabel(), 0);
				}
				if (activity.equals(duplicateActivity)) {
					for (PetrinetEdge<?, ?> edge : net.getInEdges(transition)) {
						for (Collection<PetrinetEdge<?, ?>> cluster : decomposition) {
							if (cluster.contains(edge)) {
								newCluster.addAll(cluster);
								oldClusters.add(cluster);
							}
						}
					}
					for (PetrinetEdge<?, ?> edge : net.getOutEdges(transition)) {
						for (Collection<PetrinetEdge<?, ?>> cluster : decomposition) {
							if (cluster.contains(edge)) {
								newCluster.addAll(cluster);
								oldClusters.add(cluster);
							}
						}
					}
				}
			}
			decomposition.removeAll(oldClusters);
			decomposition.add(newCluster);
		}
		/*
		 * We now have the maximal decomposition. Return it.
		 */
		return decomposition;
	}

	private Collection<Collection<PetrinetEdge<?, ?>>> getNonMaximalDecomposition(AcceptingPetriNet acceptingNet,
			Collection<Collection<PetrinetEdge<?, ?>>> decomposition,
			DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		int targetSize = Math.max(1, (decomposition.size() * getPercentage(parameters)) / 100);
		Map<Collection<PetrinetEdge<?, ?>>, Double> scoreMap = new HashMap<Collection<PetrinetEdge<?, ?>>, Double>();
		for (Collection<PetrinetEdge<?, ?>> cluster : decomposition) {
			scoreMap.put(cluster, score(cluster));
		}
		while (decomposition.size() > targetSize) {
			//			System.out.println("[DecompositionAbstractGenericStrategy] Merging " + decomposition.size() + " clusters to " + targetSize + " cluster(s).");
			double bestScore = -1.0;
			Collection<PetrinetEdge<?, ?>> bestFirstCluster = null;
			Collection<PetrinetEdge<?, ?>> bestSecondCluster = null;
			Collection<PetrinetEdge<?, ?>> bestCluster = null;
			Collection<Collection<PetrinetEdge<?, ?>>> remainingClusters = new HashSet<Collection<PetrinetEdge<?, ?>>>(
					decomposition);
			for (Collection<PetrinetEdge<?, ?>> firstCluster : decomposition) {
				double sourceScore = scoreMap.get(firstCluster);
				remainingClusters.remove(firstCluster);
				for (Collection<PetrinetEdge<?, ?>> secondCluster : remainingClusters) {
					double targetScore = scoreMap.get(secondCluster);
					Collection<PetrinetEdge<?, ?>> cluster = new HashSet<PetrinetEdge<?, ?>>();
					cluster.addAll(firstCluster);
					cluster.addAll(secondCluster);
					double score = score(cluster);
					//					System.out.println("[DecompositionAbstractGenericStrategy] Score: " + score);
					if (2 * score - sourceScore - targetScore > bestScore) {
						bestScore = score;
						bestFirstCluster = firstCluster;
						bestSecondCluster = secondCluster;
						bestCluster = cluster;
					}
				}
			}
			decomposition.remove(bestFirstCluster);
			decomposition.remove(bestSecondCluster);
			decomposition.add(bestCluster);
			scoreMap.remove(bestFirstCluster);
			scoreMap.remove(bestSecondCluster);
			scoreMap.put(bestCluster, bestScore);
		}
		//		System.out.println("[DecompositionAbstractGenericStrategy] Done merging.");
		return decomposition;
	}

	private double score(Collection<PetrinetEdge<?, ?>> cluster) {
		return interfaceMetric(cluster) * sizeMetric(cluster);
	}

	private double interfaceMetric(Collection<PetrinetEdge<?, ?>> cluster) {
		Set<Transition> inputs = new HashSet<Transition>();
		Set<Transition> outputs = new HashSet<Transition>();
		for (PetrinetEdge<?, ?> edge : cluster) {
			if (edge instanceof Arc) {
				Arc arc = (Arc) edge;
				if (arc.getSource() instanceof Transition) {
					Transition transition = (Transition) arc.getSource();
					if (!transition.isInvisible()) {
						inputs.add(transition);
					}
				} else {
					Transition transition = (Transition) arc.getTarget();
					if (!transition.isInvisible()) {
						outputs.add(transition);
					}
				}
			}
		}
		Set<Transition> internal = new HashSet<Transition>();
		internal.addAll(inputs);
		internal.retainAll(outputs);
		inputs.removeAll(internal);
		outputs.removeAll(internal);

		return 1.0 - ((1.0 * (inputs.size() + outputs.size())) / (inputs.size() + outputs.size() + internal.size()));
	}

	private double sizeMetric(Collection<PetrinetEdge<?, ?>> cluster) {
		return Math.pow(0.95, cluster.size());
	}

	private AcceptingPetriNetArray getNets(AcceptingPetriNet acceptingNet,
			Collection<Collection<PetrinetEdge<?, ?>>> decomposition,
			DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters,
			List<Map<PetrinetNode, PetrinetNode>> parentChildMaps) {
		TransEvClassMapping tecMap = parameters.getMapping();
		/*
		 * We now have decomposed all edges into valid clusters. Create an array
		 * of subnets from this.
		 */
		//		System.out.println("[DecompositionAbstractGenericStrategy] Creating subnets.");
		AcceptingPetriNetArray decomposedNets = AcceptingPetriNetArrayFactory.createAcceptingPetriNetArray();
		boolean isFirst = true;
		for (Collection<PetrinetEdge<?, ?>> cluster : decomposition) {
			//			System.out.println("[DecompositionAbstractGenericStrategy] Creating subnet for cluster " + cluster);
			AcceptingPetriNet decomposedNet = AcceptingPetriNetFactory.createAcceptingPetriNet(PetrinetFactory
					.newPetrinet(cluster.toString()));
			Map<PetrinetNode, PetrinetNode> map = new HashMap<PetrinetNode, PetrinetNode>();
			for (PetrinetEdge<?, ?> edge : cluster) {
				if (!map.containsKey(edge.getSource())) {
					if (edge.getSource() instanceof Place) {
						Place oldPlace = (Place) edge.getSource();
						Place newPlace = decomposedNet.getNet().addPlace(oldPlace.getLabel());
						map.put(oldPlace, newPlace);
					} else {
						Transition oldTransition = (Transition) edge.getSource();
						Transition newTransition = decomposedNet.getNet().addTransition(oldTransition.getLabel());
						newTransition.setInvisible(oldTransition.isInvisible());
						map.put(oldTransition, newTransition);
						tecMap.put(newTransition, tecMap.get(oldTransition));
					}
				}
				if (!map.containsKey(edge.getTarget())) {
					if (edge.getTarget() instanceof Place) {
						Place oldPlace = (Place) edge.getTarget();
						Place newPlace = decomposedNet.getNet().addPlace(oldPlace.getLabel());
						map.put(oldPlace, newPlace);
					} else {
						Transition oldTransition = (Transition) edge.getTarget();
						Transition newTransition = decomposedNet.getNet().addTransition(oldTransition.getLabel());
						newTransition.setInvisible(oldTransition.isInvisible());
						map.put(oldTransition, newTransition);
						tecMap.put(newTransition, tecMap.get(oldTransition));
					}
				}
				if (edge.getSource() instanceof Place) {
					decomposedNet.getNet().addArc((Place) map.get(edge.getSource()),
							(Transition) map.get(edge.getTarget()));
				} else {
					decomposedNet.getNet().addArc((Transition) map.get(edge.getSource()),
							(Place) map.get(edge.getTarget()));
				}
			}
			if (isFirst) {
				/*
				 * Check for disconnected transitions, as they are not in any
				 * subnet yet. Put all in first subnet.
				 */
				for (Transition transition : acceptingNet.getNet().getTransitions()) {
					if (acceptingNet.getNet().getInEdges(transition).isEmpty()
							&& acceptingNet.getNet().getOutEdges(transition).isEmpty()) {
						Transition newTransition = decomposedNet.getNet().addTransition(transition.getLabel());
						newTransition.setInvisible(transition.isInvisible());
						map.put(transition, newTransition);
						tecMap.put(newTransition, tecMap.get(transition));
					}
				}
				isFirst = false;
			}
			//			System.out.println("[DecompositionAbstractGenericStrategy] Creating inital marking cluster " + cluster);
			Marking initialMarking = new Marking();
			for (Place place : acceptingNet.getInitialMarking().baseSet()) {
				if (map.containsKey(place)) {
					initialMarking.add((Place) map.get(place), acceptingNet.getInitialMarking().occurrences(place));
				}
			}
			decomposedNet.setInitialMarking(initialMarking);
			//			System.out.println("[DecompositionAbstractGenericStrategy] Creating final markings for cluster " + cluster);
			Set<Marking> finalMarkings = new HashSet<Marking>();
			for (Marking oldFinalMarking : acceptingNet.getFinalMarkings()) {
				Marking finalMarking = new Marking();
				for (Place place : oldFinalMarking.baseSet()) {
					if (map.containsKey(place)) {
						finalMarking.add((Place) map.get(place), oldFinalMarking.occurrences(place));
					}
				}
				finalMarkings.add(finalMarking);
			}
			decomposedNet.setFinalMarkings(finalMarkings);
			decomposedNets.addNet(decomposedNet);
			parentChildMaps.add(map);
		}
		//		System.out.println("[DecompositionAbstractGenericStrategy] Done creating.");
		return decomposedNets;
	}
}
