package org.processmining.acceptingpetrinetdecomposer.strategies.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetArrayFactory;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinetdecomposer.parameters.DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters;
import org.processmining.acceptingpetrinetdecomposer.strategies.DecompositionStrategy;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;

public abstract class DecompositionAbstractStrategy implements DecompositionStrategy {

	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		return apply(context, acceptingNet, clusters, parameters, new ArrayList<Map<PetrinetNode, PetrinetNode>>());
	}

	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters,
			List<Map<PetrinetNode, PetrinetNode>> parentChildMaps) {
		AcceptingPetriNetArray acceptingNets = AcceptingPetriNetArrayFactory.createAcceptingPetriNetArray();
		acceptingNets.init();
		for (Set<XEventClass> activities : clusters.getClusters()) {
			Map<PetrinetNode, PetrinetNode> parentChildMap = new HashMap<PetrinetNode, PetrinetNode>();
			AcceptingPetriNet acceptingSubNet = apply(context, acceptingNet, activities,
					clusters.getInputs(activities), clusters.getOutputs(activities), parameters, parentChildMap);
			acceptingNets.addNet(acceptingSubNet);
			parentChildMaps.add(parentChildMap);
		}
		return acceptingNets;
	}

	private AcceptingPetriNet apply(PluginContext context, AcceptingPetriNet acceptingNet, Set<XEventClass> activities,
			Set<XEventClass> inputs, Set<XEventClass> outputs,
			DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters,
			Map<PetrinetNode, PetrinetNode> parentChildMap) {
		Petrinet net = acceptingNet.getNet();
		Petrinet subNet = PetrinetFactory.newPetrinet(net.getLabel());
		Set<Transition> forbiddenTransitions = new HashSet<Transition>();
		for (Transition transition : net.getTransitions()) {
			XEventClass activity = parameters.getMapping().get(transition);
			if (!activity.equals(parameters.getInvisibleActivity())) {
				forbiddenTransitions.add(transition);
			}
		}
		Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = new HashSet<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>>();
		Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = new HashSet<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>>();
		Set<Transition> transitions = new HashSet<Transition>();
		for (Transition transition : net.getTransitions()) {
			XEventClass activity = parameters.getMapping().get(transition);
			if (!activity.equals(parameters.getInvisibleActivity()) && activities.contains(activity)) {
				if (parameters.isIncludeClusterTransitions()) {
					transitions.add(transition);
				}
				if (outputs.contains(activity)) {
					addPredecessorEdges(net, net.getInEdges(transition), inEdges, forbiddenTransitions);
				}
				if (inputs.contains(activity)) {
					addSuccessorEdges(net, net.getOutEdges(transition), outEdges, forbiddenTransitions);
				}
			}
		}
		Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = new HashSet<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>>();
		Set<Place> places = new HashSet<Place>();

		/*
		 * The transitions in the selected subnet should not be reduced!
		 */
		Set<Transition> sacredTransitions = new HashSet<Transition>();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : inEdges) {
			if (edge.getSource() instanceof Transition) {
				sacredTransitions.add((Transition) edge.getSource());
			} else {
				sacredTransitions.add((Transition) edge.getTarget());
			}
		}
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : outEdges) {
			if (edge.getSource() instanceof Transition) {
				sacredTransitions.add((Transition) edge.getSource());
			} else {
				sacredTransitions.add((Transition) edge.getTarget());
			}
		}

		apply(context, net, transitions, places, edges, inEdges, outEdges);

		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>();
		Map<Place, Place> placeMap = new HashMap<Place, Place>();
		for (Transition transition : transitions) {
			Transition subTransition = subNet.addTransition(transition.getLabel());
			XEventClass activity = parameters.getMapping().get(transition);
			if (parameters.getMapping().get(transition).equals(parameters.getInvisibleActivity())) {
				subTransition.setInvisible(true);
			} else if (!activities.contains(activity)) {
				subTransition.setInvisible(true);
			}
			transitionMap.put(transition, subTransition);
			parentChildMap.put(transition, subTransition);
		}
		/*
		 * Set the set of transitions not-to-reduce. See comment above.
		 */
		Set<Transition> subSacredTransitions = new HashSet<Transition>();
		for (Transition sacredTransition : sacredTransitions) {
			subSacredTransitions.add(transitionMap.get(sacredTransition));
		}
		//		System.out.println("[DecompositionAbstractStrategy] Sacred: " + subSacredTransitions);
		for (Place place : places) {
			Place subPlace = subNet.addPlace(place.getLabel());
			placeMap.put(place, subPlace);
			parentChildMap.put(place, subPlace);
		}
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
			if (places.contains(edge.getSource())) {
				subNet.addArc(placeMap.get(edge.getSource()), transitionMap.get(edge.getTarget()));
			} else {
				subNet.addArc(transitionMap.get(edge.getSource()), placeMap.get(edge.getTarget()));
			}
		}
		Marking subInitialMarking = new Marking();
		for (Place place : acceptingNet.getInitialMarking().baseSet()) {
			if (places.contains(place)) {
				subInitialMarking.add(placeMap.get(place), acceptingNet.getInitialMarking().occurrences(place));
			}
		}
		Set<Marking> subFinalMarkings = new HashSet<Marking>();
		for (Marking finalMarking : acceptingNet.getFinalMarkings()) {
			Marking subFinalMarking = new Marking();
			for (Place place : finalMarking.baseSet()) {
				if (places.contains(place)) {
					subFinalMarking.add(placeMap.get(place), finalMarking.occurrences(place));
				}
			}
			subFinalMarkings.add(subFinalMarking);

		}
		AcceptingPetriNet subAcceptingPetriNet = AcceptingPetriNetFactory.createAcceptingPetriNet();
		subAcceptingPetriNet.init(subNet);
		subAcceptingPetriNet.setInitialMarking(subInitialMarking);
		subAcceptingPetriNet.setFinalMarkings(subFinalMarkings);

		return reduce(context, subAcceptingPetriNet, subSacredTransitions, parentChildMap);
	}

	private void addPredecessorEdges(Petrinet net,
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> toDo,
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges,
			Set<Transition> forbiddenTransitions) {
		while (!toDo.isEmpty()) {
			PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge = toDo.iterator().next();
			toDo.remove(edge);
			if (!edges.contains(edge)) {
				edges.add(edge);
				if (!forbiddenTransitions.contains(edge.getSource())) {
					toDo.addAll(net.getInEdges(edge.getSource()));
				}
			}
		}
	}

	private void addSuccessorEdges(Petrinet net,
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> toDo,
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges,
			Set<Transition> forbiddenTransitions) {
		while (!toDo.isEmpty()) {
			PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge = toDo.iterator().next();
			toDo.remove(edge);
			if (!edges.contains(edge)) {
				edges.add(edge);
				if (!forbiddenTransitions.contains(edge.getTarget())) {
					toDo.addAll(net.getOutEdges(edge.getTarget()));
				}
			}
		}
	}

	public abstract void apply(PluginContext context, Petrinet net, Set<Transition> transitions, Set<Place> places,
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges,
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges,
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges);

	public abstract AcceptingPetriNet reduce(PluginContext context, AcceptingPetriNet apn,
			Set<Transition> sacredTransitions, Map<PetrinetNode, PetrinetNode> parentChildMapping);

}
