package org.processmining.acceptingpetrinetdecomposer.strategies.impl;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.reduction.Murata;
import org.processmining.plugins.petrinet.reduction.MurataInput;
import org.processmining.plugins.petrinet.reduction.MurataOutput;
import org.processmining.plugins.petrinet.reduction.MurataParameters;

public class DecompositionReplaceReduceStrategy extends DecompositionAbstractStrategy {

	public final static String NAME = "Make invisible w/ Reduction";

	public void apply(PluginContext context, Petrinet net, Set<Transition> transitions, Set<Place> places,
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges,
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges,
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges) {
		edges.addAll(net.getEdges());
		places.addAll(net.getPlaces());
		transitions.addAll(net.getTransitions());
	}

	public AcceptingPetriNet reduce(PluginContext context, AcceptingPetriNet net, Set<Transition> sacredTransitions,
			Map<PetrinetNode, PetrinetNode> parentChildMapping) {
		Murata murata = new Murata();
		MurataInput input = new MurataInput(net.getNet(), net.getInitialMarking());
		//		input.setVisibleSacred(net.getNet());
		System.out.println("[DecompositionReplaceReduceStrategy] Sacred: " + sacredTransitions);
		for (Transition sacredTransition : sacredTransitions) {
			input.addSacred(sacredTransition);
		}
		for (Place place : net.getInitialMarking().baseSet()) {
			input.addSacred(place);
		}
		for (Marking finalMarking : net.getFinalMarkings()) {
			for (Place place : finalMarking.baseSet()) {
				input.addSacred(place);
			}
		}
		input.allowRule(MurataInput.CSM);
		input.allowRule(MurataInput.ASM);
		MurataOutput output;
		MurataParameters parameters = new MurataParameters();
		parameters.setAllowFPTSacredNode(false);
		try {
			output = murata.run(context, input, parameters);
		} catch (ConnectionCannotBeObtained e) {
			// Reduction failed. Return original net.
			return net;
		}
		AcceptingPetriNet reducedNet = AcceptingPetriNetFactory.createAcceptingPetriNet();
		reducedNet.init(output.getNet());
		Map<Place, Place> placeMap = output.getPlaceMapping();
		Marking subInitialMarking = new Marking();
		for (Place place : net.getInitialMarking().baseSet()) {
			subInitialMarking.add(placeMap.get(place), net.getInitialMarking().occurrences(place));
		}
		reducedNet.setInitialMarking(subInitialMarking);
		Set<Marking> subFinalMarkings = new HashSet<Marking>();
		for (Marking finalMarking : net.getFinalMarkings()) {
			Marking subFinalMarking = new Marking();
			for (Place place : finalMarking.baseSet()) {
				subFinalMarking.add(placeMap.get(place), finalMarking.occurrences(place));
			}
			subFinalMarkings.add(subFinalMarking);
		}
		reducedNet.setFinalMarkings(subFinalMarkings);
		/*
		 * The following fragment aims to update the parentChildMapping. However, the maps obtained
		 * from the reduction do not really contain sufficient information. Basically, these map contain
		 * information on by which node a node was replaced, but many nodes could have been replaced by one.
		 */
		Map<Transition, Transition> transitionMap = output.getTransitionMapping();
		for (PetrinetNode originalNode : new HashSet<PetrinetNode>(parentChildMapping.keySet())) {
			PetrinetNode decomposedNode = parentChildMapping.get(originalNode);

			if (placeMap.get(decomposedNode) != null
					&& placeMap.get(decomposedNode).getLabel().equals(decomposedNode.getLabel())
					&& reducedNet.getNet().getPlaces().contains(placeMap.get(decomposedNode))) {
				/*
				 * Place has probably survived the reduction. Update to place in the
				 * reduced net.
				 */
				parentChildMapping.put(originalNode, placeMap.get(decomposedNode));
			} else if (transitionMap.get(decomposedNode) != null
					&& transitionMap.get(decomposedNode).getLabel().equals(decomposedNode.getLabel())
					&& reducedNet.getNet().getTransitions().contains(transitionMap.get(decomposedNode))) {
				/*
				 * Transition has probably survived the reduction. Update to transition
				 * in the reduced net.
				 */
				parentChildMapping.put(originalNode, transitionMap.get(decomposedNode));
			} else {
				/*
				 * Node has probably not survived the reduction. Remove.
				 */
				parentChildMapping.remove(originalNode);
			}
		}
		return reducedNet;
	}

	public String getName() {
		return NAME;
	}
}
