package org.processmining.acceptingpetrinetdecomposer.strategies.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinetdecomposer.parameters.DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters;
import org.processmining.acceptingpetrinetdecomposer.parameters.sese.DecomposeBySESEsWithBridgingParameters;
import org.processmining.acceptingpetrinetdecomposer.plugins.sese.DecomposeBySESEsWithBridgingPlugin;
import org.processmining.acceptingpetrinetdecomposer.strategies.DecompositionStrategy;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

public class DecompositionSESEStrategy implements DecompositionStrategy {

	public static final String NAME = "SESE-based";
	
	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		return apply(context, acceptingNet, clusters, parameters, new ArrayList<Map<PetrinetNode, PetrinetNode>>());
	}

	public AcceptingPetriNetArray apply(PluginContext context, AcceptingPetriNet acceptingNet,
			ActivityClusterArray clusters, DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters,
			List<Map<PetrinetNode, PetrinetNode>> parentChildMaps) {
		AcceptingPetriNetArray acceptingNetArray = getSESEDecomposition(context, acceptingNet, parameters);
		return acceptingNetArray;
	}
	
	public AcceptingPetriNetArray getSESEDecomposition(PluginContext context, AcceptingPetriNet acceptingNet,
			DecomposeAcceptingPetriNetUsingActivityClusterArrayParameters parameters) {
		DecomposeBySESEsWithBridgingParameters params = new DecomposeBySESEsWithBridgingParameters();
		params.setMaxSize(parameters.getMaxSubnetArcs());		
		// get the unsplittable transitions
		Set<Transition> unsplittables = getUnsplittables(acceptingNet.getNet(), parameters.getUnsplittableActivities(), 
				parameters.getMapping());
		params.setUnsplittables(unsplittables);
		System.out.println("[DecompositionSESEStrategy] Unsplittables: " + unsplittables);
		
		DecomposeBySESEsWithBridgingPlugin plugin = new DecomposeBySESEsWithBridgingPlugin();
		AcceptingPetriNetArray acceptingNetArray = plugin.run(context, acceptingNet, params);
		context.getProvidedObjectManager().createProvidedObject(acceptingNetArray.toString(), 
				acceptingNetArray, context);
 		
		System.out.println("[DecompositionSESEStrategy] No. of subnets: " + acceptingNetArray.getSize());
		
		return acceptingNetArray;
	}
	
	private Set<Transition> getUnsplittables(Petrinet net, Set<XEventClass> unsplittableActs, 
			TransEvClassMapping tecMap) {
		Set<Transition> unsplittables = new HashSet<>();
		for (Transition transition: net.getTransitions()) {
			XEventClass activity = tecMap.get(transition);
			if (unsplittableActs.contains(activity)) {
				unsplittables.add(transition);
			}
		}
		return unsplittables;
	}

	public String getName() {
		return NAME;
	}

}
