package org.processmining.acceptingpetrinetdecomposer.utils.dc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinetdecomposer.models.rpst.PetriNetRPSTNode;
import org.processmining.acceptingpetrinetdecomposer.utils.petrinet.PetriNetUtils;
import org.processmining.acceptingpetrinetdecomposer.utils.rpst.PetriNetRPSTUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;

public class BridgingUtils {
	
	public static List<Collection<PetrinetEdge<?, ?>>> bridging(List<PetriNetRPSTNode> nodes, 
			AcceptingPetriNet apn) {
		List<Collection<PetrinetEdge<?, ?>>> decomposition = new ArrayList<Collection<PetrinetEdge<?, ?>>>();
		
		Petrinet net = apn.getNet();
		
		Set<Place> startPlace = PetriNetUtils.getStartPlaces(net);
		Set<Place> endPlace = PetriNetUtils.getEndPlaces(net);
		
		Set<Place> entryPlaces = PetriNetRPSTUtils.getEntryPlaces(nodes);
		Set<Place> exitPlaces = PetriNetRPSTUtils.getExitPlaces(nodes);
		
		Set<Place> sharedPlaces = new HashSet<>();
		sharedPlaces.addAll(entryPlaces);
		sharedPlaces.addAll(exitPlaces);
		sharedPlaces.removeAll(startPlace);
		sharedPlaces.removeAll(endPlace);
		
		// create sets of edges where each set is a subcomponent
		for (PetriNetRPSTNode node: nodes) {
			// new cluster
			Collection<PetrinetEdge<?, ?>> cluster = new HashSet<>();
			
			// cluster should not have any arcs that are connected to a shared place
			for (Arc arc: node.getArcs()) {
				if (!sharedPlaces.contains(arc.getSource()) &&
						!sharedPlaces.contains(arc.getTarget())) 
					cluster.add(arc);
			}
			
			if (cluster.size() > 0) 
				decomposition.add(cluster);
		}
		
		// each shared place is a bridge which is also a subcomponent
		for (Place sharedPlace: sharedPlaces) {
			// new cluster
			Collection<PetrinetEdge<?, ?>> cluster = new HashSet<>();
			
			cluster.addAll(net.getInEdges(sharedPlace));
			cluster.addAll(net.getOutEdges(sharedPlace));
			
			if (cluster.size() > 0) 
				decomposition.add(cluster);
		}
		
		return decomposition;
	}
	
}
