package org.processmining.acceptingpetrinetdecomposer.utils.petrinet;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetArrayFactory;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;

public class AcceptingPetriNetArrayUtils {

	public static AcceptingPetriNetArray getNets(AcceptingPetriNet apn,
			List<Collection<PetrinetEdge<?, ?>>> decomposition,
			List<Map<PetrinetNode, PetrinetNode>> parentChildMaps) {
		AcceptingPetriNetArray decomposedNets = AcceptingPetriNetArrayFactory.createAcceptingPetriNetArray();
		
		boolean isFirst = true;
		
		for (Collection<PetrinetEdge<?, ?>> cluster: decomposition) {
			Petrinet newNet = PetrinetFactory.newPetrinet(cluster.toString());
			// new accepting petri net for the cluster
			AcceptingPetriNet decomposedNet = AcceptingPetriNetFactory.createAcceptingPetriNet(newNet);
			Map<PetrinetNode, PetrinetNode> map = new HashMap<PetrinetNode, PetrinetNode>();
			
			for (PetrinetEdge<?, ?> edge: cluster) {
				// adding all the places and transitions connected to these edges
				if (!map.containsKey(edge.getSource())) {
					if (edge.getSource() instanceof Place) {
						Place oldPlace = (Place) edge.getSource();
						Place newPlace = newNet.addPlace(oldPlace.getLabel());
						newPlace.setLocalID(oldPlace.getLocalID());
						map.put(oldPlace, newPlace);
					} else {
						Transition oldTransition = (Transition) edge.getSource();
						Transition newTransition = newNet.addTransition(oldTransition.getLabel());
						newTransition.setInvisible(oldTransition.isInvisible());
						newTransition.setLocalID(oldTransition.getLocalID());
						map.put(oldTransition, newTransition);
					}
				}
				if (!map.containsKey(edge.getTarget())) {
					if (edge.getTarget() instanceof Place) {
						Place oldPlace = (Place) edge.getTarget();
						Place newPlace = newNet.addPlace(oldPlace.getLabel());
						newPlace.setLocalID(oldPlace.getLocalID());
						map.put(oldPlace, newPlace);
					} else {
						Transition oldTransition = (Transition) edge.getTarget();
						Transition newTransition = newNet.addTransition(oldTransition.getLabel());
						newTransition.setInvisible(oldTransition.isInvisible());
						newTransition.setLocalID(oldTransition.getLocalID());
						map.put(oldTransition, newTransition);
					}
				}
				if (edge.getSource() instanceof Place) {
					newNet.addArc((Place) map.get(edge.getSource()), 
							(Transition) map.get(edge.getTarget()));					
				} else {
					newNet.addArc((Transition) map.get(edge.getSource()), 
							(Place) map.get(edge.getTarget()));
				}
			}
			
			if (isFirst) {
				// Check for disconnected transitions, as they are not in any subnet yet. 
				// Put all in first subnet.
				for (Transition transition: apn.getNet().getTransitions()) {
					if (apn.getNet().getInEdges(transition).isEmpty() &&
							apn.getNet().getOutEdges(transition).isEmpty()) {
						Transition newTransition = decomposedNet.getNet().addTransition(transition.getLabel());
						newTransition.setInvisible(transition.isInvisible());
						newTransition.setLocalID(transition.getLocalID());
					}
				}
				isFirst = false;
			}
			
			Marking initialMarking = new Marking();
			for (Place place: apn.getInitialMarking().baseSet()) {
				if (map.containsKey(place)) 
					initialMarking.add((Place) map.get(place), apn.getInitialMarking().occurrences(place));
			}
			decomposedNet.setInitialMarking(initialMarking);
			
			Set<Marking> finalMarkings = new HashSet<>();
			for (Marking oldFinalMarking: apn.getFinalMarkings()) {
				Marking finalMarking = new Marking();
				for (Place place: oldFinalMarking.baseSet()) {
					if (map.containsKey(place)) 
						finalMarking.add((Place) map.get(place), oldFinalMarking.occurrences(place));
				}
				finalMarkings.add(finalMarking);
			}
			decomposedNet.setFinalMarkings(finalMarkings);
			decomposedNets.addNet(decomposedNet);
			parentChildMaps.add(map);
		}
		return decomposedNets;
	}
	
}
